

#ifndef __IMAGER_WVLTCORE_H__
#define __IMAGER_WVLTCORE_H__

#include "defs.h"



/* Different mother wavelet functions implemented here */

typedef enum { W_NULL, W_HAAR, W_DAUB4, W_DAUB8, W_DAUB12, W_DAUB20, W_COIF6, W_COIF12, W_SYM4, W_SYM8, W_SYM12 } WAVELET_TYPE;

/* Forward or inverse transform, just mnemonics. */

typedef enum { W_FORWARD, W_INVERSE } DWTDIRECTION;



/* This is the structure that holds a subband filter */

typedef struct 
{
    int n_param;		/* Number of filter coefficients */
    int hk_offs;		/* Parameter offset for scale filter centering, usually -n_param/2 */
    int gk_offs;		/* Parameter offset for wavelet filter centering, usually -n_param/2 */
    float *hk;		/* Array of scale filter coefficients */
    float *gk;		/* Array of wavelet filter coefficients */
} 
wavefilt;


/* A subband filter function, needs to point at the dyadic subband decomposition function */

void (*subband)(float *, unsigned long, DWTDIRECTION, wavefilt);	/* Pointer to the selected subband filter function */


char wvlt_used[32];			/* For messaging/debugging purposes */


/********************************************************/



wavefilt wavelet_prepare (WAVELET_TYPE wlt);
void general_subband (float *data, unsigned long n, DWTDIRECTION idwt, wavefilt wfilt);
void dead_subband (float *data, unsigned long n, DWTDIRECTION idwt, wavefilt dummy);
WAVELET_TYPE set_subband (int wavelet);

void dwt_1d (float *data, unsigned long n, DWTDIRECTION idwt, wavefilt wfilt, void (*wt_subband)(float [], unsigned long, DWTDIRECTION, wavefilt));
void dwt_1dn (float *data, unsigned long n, DWTDIRECTION idwt, int levels, wavefilt wfilt, void (*wt_subband)(float [], unsigned long, DWTDIRECTION, wavefilt));
void dwt_2d (image_type *img, DWTDIRECTION idwt, int decomp_level, wavefilt wfilt, void (*wt_subband)(float *, unsigned long, DWTDIRECTION, wavefilt));

float wv_thresh (float elem, float t, int hardsoft);
float estimate_sigma_1d (float *data, int len);
float estimate_sigma_2d (image_type *img, int z);

float find_1d_wsthreshold (float *data, int len, float sigma, int wshrinkfunc);
void wvlt_1D_denoising  (float *data, int len, int wfunc, int wshrinkfunc, float tmult);
void wavelet_shrink (image_type *img, int decomp_level, int wshrinkfunc, float tmult);


#endif
